"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StageFacade = void 0;
const stateService_1 = require("./services/stateService");
const drop_1 = require("./useCases/drop");
const delete_1 = require("./useCases/delete");
const export_1 = require("./useCases/export");
const import_1 = require("./useCases/import");
const truncate_1 = require("./useCases/truncate");
const sync_1 = require("./useCases/sync");
class StageFacade {
    constructor(schemaFacade, expression, executor, languages, helper) {
        this.schemaFacade = schemaFacade;
        this.expression = expression;
        this.executor = executor;
        this.languages = languages;
        this.helper = helper;
        this.stageMappingService = new stateService_1.StageMappingService(schemaFacade, this.helper);
        this.stageModelService = new stateService_1.StageModelService(schemaFacade, this.helper);
    }
    exists(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const file = this.stageModelService.getFile(name);
            return this.helper.fs.exists(file);
        });
    }
    sync(options) {
        const _options = this.expression.solveOptions(options);
        return new sync_1.StageSync(this.executor, this.stageModelService, this.schemaFacade, this.languages, _options, this.helper);
    }
    drop(options) {
        const _options = this.expression.solveOptions(options);
        return new drop_1.StageDrop(this.executor, this.stageModelService, this.stageMappingService, this.schemaFacade, this.languages, _options, this.helper);
    }
    truncate(options) {
        const _options = this.expression.solveOptions(options);
        return new truncate_1.StageTruncate(this.executor, this.stageModelService, this.schemaFacade, this.languages, _options, this.helper);
    }
    delete(options) {
        const _options = this.expression.solveOptions(options);
        return new delete_1.StageDelete(this.stageMappingService, this.schemaFacade.domain, this.expression, this.executor, _options);
    }
    export(options) {
        const _options = this.expression.solveOptions(options);
        return new export_1.StageExport(this.stageMappingService, this.schemaFacade.domain, this.expression, this.executor, _options);
    }
    import(options) {
        const _options = this.expression.solveOptions(options);
        return new import_1.StageImport(this.stageMappingService, this.schemaFacade.domain, this.expression, this.executor, _options);
    }
}
exports.StageFacade = StageFacade;
//# sourceMappingURL=facade.js.map